//-----------------------------------------------------------------------------      
void HitF(void)
{
  /* drive pin to activate the magnet
     change cycle time randomly within given period
  */
 
  unsigned int MinTime;
  unsigned int MaxTime;

  unsigned int CurrentTime;

  if (!HitStat)
  {

    // to debug cycle time ------
    CurrentTime=millis()-T;
    if (CurrentTime > (2*GotTime)) (CurrentTime=GotTime);
    
    T=millis();
  
    Ttot+=(CurrentTime);
    Minimum=min(Minimum, CurrentTime);
    Maximum=max(Maximum, CurrentTime);
    MeanTime=Ttot/Tcount;
/*
    Serial.print("Current Time: ");
    Serial.print(CurrentTime);
    Serial.print("  Minimum Time: ");
    Serial.print(Minimum);
    Serial.print("  Maximum Time: ");
    Serial.print(Maximum);
    Serial.print("  Mean Time: ");
    Serial.print(MeanTime);
    Serial.print("  Count: ");
    Serial.println(Tcount);
*/
    Tcount++;
    //---------------------------

    HitStat = true;
    HitCycle.interval(HitTimeOn);  // change to ON period
    if (PirFlag)
    {
      digitalWrite(HIT,HIGH);
    }
  }
  else
  {
    HitStat = false;
    MinTime = (GotTime / 10); 
    MaxTime = (2 * GotTime) - MinTime; 
    HitTimeOff = random(MinTime, MaxTime);
    HitCycle.interval(HitTimeOff);  // change to OFF period
    digitalWrite(HIT,LOW);
  }
}

//-----------------------------------------------------------------------------      
void LedGf(void)
{
  if (!LedGstat)
  {
    LedGstat = true;
    LedGcycle.interval(LedGtimeOn);  // change to ON period
    digitalWrite(LED_G,HIGH);
  }
  else
  {
    LedGstat = false;
    LedGcycle.interval(LedGtimeOff);  // change to OFF period
    digitalWrite(LED_G,LOW);
  }
}

//-----------------------------------------------------------------------------      
void LedRf(void)
{
  if (!LedRstat)
  {
    LedRstat = true;
    LedRcycle.interval(LedRtimeOn);  // change to ON period
    digitalWrite(LED_R,HIGH);
  }
  else
  {
    LedRstat = false;
    LedRcycle.interval(LedRtimeOff);  // change to OFF period
    digitalWrite(LED_R,LOW);
  }
}

//-----------------------------------------------------------------------------      
void GetF(void)
{  
  int i; // generic counter
  int j; // generic counter
  char TempGotTime[16];
  char TempGotTime2[8];
  
  if ((millis() - TimeStart) > TIMEOUT)
  {  // timeout, send another request
    SpiSerial.flush();
    FsmStatus = 0;
    StrPtr=0;
    RxStatus = 0;
    SpiSerial.print("$$$");
    delay(300);
    SpiSerial.print("close");
    TimeStart = millis(); // reset timeout
    Error(1);
    FsmFlag = 1;
    ErrorCount ++;
    return;
  }
  
  if(!FsmFlag)
  {
    return;
  }
  
  switch (FsmStatus)
  {     
    case 0:	// idle
      SpiSerial.println("");
      SpiSerial.print("open ");
      SpiSerial.print(WebSite);
      SpiSerial.print(" ");
      SpiSerial.println(WebPort);
      
      RxStatus = 0; // command sent, waiting confirm
      strcpy(RxString, OPEN_STRING);
      
      FsmFlag = 0;   // wait for correct decoding
      TimeStart = millis(); // reset timeout
      FsmStatus = 1; // wait for text decoding from RX
      GetCycle.interval(GetTimeFast);
    break;
    
    case 1: 
      StartVar = StopPointer; // the got variable is between end of open and begin of close
      SpiSerial.println(WebGet);
      
      RxStatus = 0; // command sent, waiting confirm
      strcpy(RxString, CLOSE_STRING);
      
      FsmFlag = 0;   // wait for correct decoding
      TimeStart = millis(); // reset timeout
      FsmStatus = 2; // next status 
      GetCycle.interval(GetTimeSlow);
    break;

    case 2:
      StopVar = StartPointer; // the got variable is between end of open and begin of close   

      i=0;
      j=StartVar;
      while (i<=MAX_RX_BUFF)
      {
        j++;
        if (j>=MAX_RX_BUFF) j=0; //reset circ queue
        
        TempGotTime[i] = RxBuff[j];
        i++;
        
        if(j==StopVar)
        {
          break;
        }
      }
 
      if((TempGotTime[0]=='*')&&(TempGotTime[1]=='-')&&(TempGotTime[i-2]=='-')&&(TempGotTime[i-1]=='*'))
      {
        for(j=0; j<=i-4; j++)
        {
          TempGotTime2[j] = TempGotTime[j+2];
        }
      }
      TempGotTime2[j-1]='\0';  // null terminator
      GotTime=atoi(TempGotTime2);
      
      if(GotTime != EeRead())
      {
        // if new data save in EEPROM
        EeSave();
      }
      
      SpiSerial.print("$$$");
      RxStatus = 0; // command sent, waiting confirm
      strcpy(RxString, "CMD");
      
      FsmFlag = 0;   // wait for correct decoding
      TimeStart = millis(); // reset timeout
      FsmStatus = 3; // next status 
      GetCycle.interval(GetTimeFast);
    break;

    case 3:
      RxStatus = 0;
      TimeStart = millis(); // reset timeout
      FsmStatus = 0; // restart 
      FsmFlag = 1; 
      GetCycle.interval(GetTimeFast);
      OkCount++;
      
      //------------- log stamp
      Serial.println("");
      Serial.print("Ok: ");
      Serial.print(OkCount);
      Serial.print("  Errors: ");
      Serial.print(ErrorCount);
      Serial.print("  GotTime: ");
      Serial.print(GotTime);
      Serial.print("  MeanTime: ");
      Serial.print(MeanTime);
      Serial.print("  PirOn: ");
      Serial.print(PirFlag, BIN);
     
      //------------- log stamp

    break;
    
    default:

    break;
    
  } // switch end
}

//-----------------------------------------------------------------------------      
void Error(int err)
{
  if (err == 0)
  {
    ErrorCountWDT = 0; // reset error count
    LedBlink(1);  // blink normally
  }
  else
  {
    ErrorCountWDT ++; // consecutive errors
    LedBlink(2);  // blink fast
  }
  
  if (ErrorCountWDT >= 15)
  {
    // too many errors: restart
    ErrorCountWDT = 0;
    WiFly.begin();
    
    while (!WiFly.join(ssid, passphrase)) 
    {
      Serial.println("Association failed.");
      delay(5000);
        // repeat continuously until associated
        // hang on failure. Check Credential.h 
    }  
   
    Serial.println("Associated."); 
  }
}

//-----------------------------------------------------------------------------      
void EeSave(void)
{
  unsigned char FirstByte;
  unsigned char SecondByte;
  
  FirstByte = GotTime & 0x00FF;   // lower byte
  SecondByte = GotTime >> 8;      // upper byte

  EEPROM.write(EeGotTimeAddr, FirstByte);
  EEPROM.write(EeGotTimeAddr+1, SecondByte);
  
  Serial.print("Saved: ");
  Serial.println(GotTime, DEC);
}

//-----------------------------------------------------------------------------      
int EeRead(void)
{
  unsigned char FirstByte;
  unsigned char SecondByte;
  int ReadValue;
  
  FirstByte =  EEPROM.read(EeGotTimeAddr);
  SecondByte = EEPROM.read(EeGotTimeAddr+1);
  
  ReadValue = FirstByte + (SecondByte << 8);
  
  if (ReadValue == 0xFFFF)
  {
    ReadValue = 30000; // if EEPROM was never written the value is 0xFF
  }
  
  return ReadValue;
}

//-----------------------------------------------------------------------------      
void Pir(void)
{
  if (digitalRead(PIR))
  {
    Seconds ++;
  }
  else
  {
    Seconds = 0;
    PirFlag = 1;
    if (!AssociatedFlag)
    {
      LedBlink(1);
    }
  }
  
  if(Seconds >= PIR_TIMEOUT)
  {
    Seconds = PIR_TIMEOUT;
    PirFlag = 0;
    LedBlink(0);  // blink slowly, no activity
  } 
}

//-----------------------------------------------------------------------------      
void LedBlink(int Blink)
{
 switch (Blink)
  {     
    case 0:	
      LedGtimeOn = 2000;
      LedGtimeOff = 2000;
      LedRtimeOn = 2000;
      LedRtimeOff = 2000;
    break;
    
    case 1:	
      LedGtimeOn = BlinkOn;
      LedGtimeOff = BlinkOff;
      LedRtimeOn = BlinkOff;
      LedRtimeOff = BlinkOn; 
    break;

    case 2:	
      LedGtimeOn = 100;
      LedGtimeOff = 100;
      LedRtimeOn = 100;
      LedRtimeOff = 100; 
    break;
    
    default:
      LedGtimeOn = 500;
      LedGtimeOff = 500;
      LedRtimeOn = 500;
      LedRtimeOff = 500; 
    break;
  }
    
}
