
/* ////////////////////////////////////////////////////////////////////////////
** File:      WiFlyHammer.pde
*/                                   
  char  Ver[] = "WiFlyHammer  0.9.3 Guiott 04-10"; // 30+1 char
/* Author:    Guido Ottaviani-->guido@guiott.com<--
** Description: Gets a data on a web server via WiFly interface to drive an
**               external input with a time proportional to the data received
**
-------------------------------------------------------------------------------
Copyright 2010 Guido Ottaviani
guido@guiott.com

These files are licensed under a:
Creative Commons Attribution Share-Alike license
-------------------------------------------------------------------------------      
/////////////////////////////////////////////////////////////////////////////*/

#include "WFH.h"
#include <MyWiFly.h>
#include <Metro.h>
#include <string.h>
#include <EEPROM.h>
#include "Credentials.h"

  // to debug cycle time ------
  unsigned long T;
  
  unsigned long Ttot;
  unsigned long Tcount = 1;
  unsigned int Minimum=4000;
  unsigned int Maximum=0;
  unsigned int MeanTime;
  //---------------------------

unsigned int FsmStatus = 0; // current state for the Finite State Machine
unsigned int GotTime = 60000; // period got from web site

#define HIT 4      // hammer driver
boolean HitStat = true;
unsigned int HitTimeOn = 50; // enough time to drive the hammer
unsigned int HitTimeOff = 1950; // current period randomly computed

#define NORMAL_BLINK_ON 500
#define NORMAL_BLINK_OFF 500
#define NOT_WEB_BLINK_ON 200
#define NOT_WEB_BLINK_OFF 800
unsigned int BlinkOn = NORMAL_BLINK_ON;
unsigned int BlinkOff = NORMAL_BLINK_OFF;
boolean AssociatedFlag = 1;

#define LED_G 5
boolean LedGstat = false;
unsigned int LedGtimeOn = NORMAL_BLINK_ON;
unsigned int LedGtimeOff = NORMAL_BLINK_OFF;

#define LED_R 6
boolean LedRstat = false;
unsigned int LedRtimeOn = NORMAL_BLINK_ON;
unsigned int LedRtimeOff = NORMAL_BLINK_OFF;

#define PIR 3  // PIR input

Client client(WebSite, 80);

Metro HitCycle = Metro(HitTimeOff,1);    // Hammer hit cycle
Metro LedGcycle = Metro(LedGtimeOff,1);  // Green led cycle
Metro LedRcycle = Metro(LedRtimeOff,1);  // Red led cycle

Metro OneSecond = Metro(1000,1);         // one second cycle

unsigned int GetTimeSlow = 5000;
unsigned int GetTimeFast = 1000;
#define TIMEOUT 15000
Metro GetCycle = Metro(GetTimeSlow,1);       // Getweb data cycle

//RX 
#define MAX_RX_BUFF 128   // less then 255, pointers are char variables
unsigned char RxBuff[MAX_RX_BUFF];    //serial communication buffer
unsigned char RxStatus = 0;  // Rx FSM
char RxString[16];  // expected string
#define CLOSE_STRING "*CLOS*"
#define OPEN_STRING  "*OPEN*"

// Rx circular queue pointers
unsigned char RxPtrIn=0;	// captured ingoing bytes 
unsigned char RxPtrOut=0;	// read outgoing bytes 
unsigned char RxPtrStart=0;	// message packet starting pointer in queue
unsigned char RxPtrEnd=0;	// message packet ending pointer in queue
unsigned char RxPtrData=0;	// pointer to first data in queue
unsigned char TmpPtr = 0;	// temporary pointer
unsigned char StrPtr = 0;       // string pointer
unsigned char StartPointer;     // string start
unsigned char StopPointer;      // string stop
unsigned char StartVar;         // to find variable string
unsigned char StopVar;          // to find variable string

unsigned long TimeStart;
boolean FsmFlag = 1;
unsigned int ErrorCount = 0;
unsigned int OkCount = 0;
unsigned int ErrorCountWDT = 0;

int EeGotTimeAddr = 0;  // address of the variable GotTime stored in EEPROM
int Seconds = 0; // time elapsed for PIR timeout
#define PIR_TIMEOUT 30
boolean PirFlag = 1;
//-----------------------------------------------------------------------------      
void setup() 
{
  int i; // generic counter
  #define MAX_ATTEMPT 5
  pinMode(HIT, OUTPUT); // Led Green
  pinMode(LED_G, OUTPUT); // Led Red
  pinMode(LED_R, OUTPUT); // Driver
  digitalWrite(HIT, LOW);
  digitalWrite(LED_G, HIGH);
  digitalWrite(LED_R, HIGH);
  
  pinMode(PIR, INPUT);  // PIR input
  
  Serial.begin(9600);
  
  for (i=0; i<strlen(Ver); i++)
  {
    Serial.print(Ver[i], BYTE);
  }
  Serial.println();
  
  WiFly.begin();

if(!TerminalMode)
{
  // test mode configured in "Credential.h" file
  i = 0;
  while ((!WiFly.join(ssid, passphrase,Wap) && (i < MAX_ATTEMPT))) 
  {
    // if not associated to a valid network after MAX_ATTEMPT attempts, 
    // start without updating cycle time from web page, 
    // the last valid read data will be used
 
    i++;  
    Serial.print("Association to: ");
    Serial.print(ssid);
    Serial.print(" failed # ");
    Serial.println(i, DEC);
    
    delay(5000);
      // repeat continuously until associated
      // hang on failure. Check Credential.h 
  }  
 
  if (i >= MAX_ATTEMPT)
  {
    Serial.print("Not Associated. Proceede without updating data: ");
    BlinkOn  = NOT_WEB_BLINK_ON;
    BlinkOff = NOT_WEB_BLINK_OFF;
    LedBlink(1);
    AssociatedFlag = 0;
  }
  else
  {
    Serial.print("Associated to: "); 
    Serial.println(ssid); 
    BlinkOn  = NORMAL_BLINK_ON;
    BlinkOff = NORMAL_BLINK_OFF;
    AssociatedFlag = 1;
  }
}
else
{
  Serial.println("Entering in terminal mode");
}
  
  digitalWrite(LED_R, LOW);
  LedGstat = true;
  
  GotTime = EeRead();  // initialize hit interval with the last succesfully read value
  Serial.println(GotTime);
}

//-----------------------------------------------------------------------------      
void loop() 
{
  if (HitCycle.check()) {HitF();}   // hit the hammer
  if (LedGcycle.check()) {LedGf();} // led green
  if (LedRcycle.check()) {LedRf();} // led red
  if(!TerminalMode && AssociatedFlag)
  {  
    if (GetCycle.check()) {GetF();}   // get data from web
  }
  
  if (OneSecond.check()) {Pir();}   // PIR reset
  
  if (SpiSerial.available()) {ReadChar();}
  if (RxPtrIn != RxPtrOut) {Rx();}
  
  if(Serial.available()) 
  { // Send out data typed on terminal
    SpiSerial.print(Serial.read(), BYTE);
  }
}


